// timer.h 
//  Data structures to emulate a hardware timer.
//
//  A hardware timer generates a CPU interrupt every X milliseconds.
//  This means it can be used for implementing time-slicing.
//
//  We emulate a hardware timer by scheduling an interrupt to occur
//  every time stats->totalTicks has increased by TimerTicks.
//
//  In order to introduce some randomness into time-slicing, if "doRandom"
//  is set, then the interrupt is comes after a random number of ticks.
//
//  DO NOT CHANGE -- part of the machine emulation
//
// Copyright (c) 1992-1993 The Regents of the University of California.
// All rights reserved.  See copyright.h for copyright notice and limitation 
// of liability and disclaimer of warranty provisions.

#ifndef TIMER_H
#define TIMER_H

#ifndef HOST_DOS
#include "copyright.h"
#include "utility.h"
#else
#include "copyrght.hpp"
#include "utility.hpp"
#endif

// The following class defines a hardware timer. 
class Timer {
  public:
    Timer(VoidFunctionPtr timerHandler, long callArg, bool doRandom);
    ~Timer() {}

// these
    void TimerExpired();    // called internally when the hardware
                // timer generates an interrupt

    long TimeOfNextInterrupt();  // figure out when the timer will generate
                // its next interrupt 

  private:
    bool randomize;     // if we are to choose a random timeout delay
    VoidFunctionPtr handler;    // code to invoke to handle timer interrupt
    long arg;            // argument to pass to interrupt handler

};

#endif // TIMER_H
