// bridge.cc - implementation of HW1 problem 6


//         ***GENERAL IDEA***
// 
//  Unless there are no waiters cars queue up on either side of
//  the bridge as they arrive.  Cars waiting to go the current 
//  direction have priority.  The direction of travel on the 
//  bridge does not change until all cars have crossed.


#include "bridge.h"

extern "C" {
#include <stdlib.h>
#include <time.h>
}

#include "alarm.h"
#include "system.h"
#include "stats.h"
#include "utility.h"
#include "thread.h"


Bridge* bridge = new Bridge();

void ReleaseCars(int i);
direction SwitchD(direction d);

static char *threadNames[]= {"first", "second", "third", "fourth",
       "fifth", "sixth", "seventh", "eighth", "ninth", "tenth",
       "eleventh", "twelfth", "thirteenth", "fourteenth",
       "fifteenth", "sixteenth", "seventeenth", "eighteenth",
       "nineteenth", "twentieth", "twentyfirst", "twentysecond",
       "twenty-third", "twenty-fourth", "twenty-fifth",
       "twenty-sixth", "twenty-seventh", "twenty-eighth", 
       "twenty-ninth", "thirtieth", "thirty-first",
       "thirty-second", "thirty-third", "thirty-fourth",
       "thirty-fifth" };

//testing purposes only--attempt to generate random input.

void ReleaseCars(int i) {
  direction d;
  srand((time(NULL) * i) % 31769);
  alarm->GoToSleepUntil(stats->totalTicks + (rand() % 203));
  i = (rand() % 10007) % 2;
  if (i) 
    d = WEST;
  else d = EAST;
  bridge->ArriveBridge(d);
}

void OpenBridge() {
  Thread *t;
  for (int i = 0; i < 35; i++)
    { t = new Thread(threadNames[i]);
      t->Fork(ReleaseCars, i);
    }
}

Bridge::Bridge() {
  bridgeLock = new Lock("Bridge Lock");
  numCarsOnBridge = numWaitingGoingEast = numWaitingGoingWest = 0;
  currentDirection = EAST;  //initialize arbitrarily 
  safeToCrossGoingWest = new Condition("Bridge Safe West");
  safeToCrossGoingEast = new Condition("Bridge Safe East");
}



//  Each car blocks on WaitInLine as it arrives unless
//  the current direction is the same as its direction
//  and there are no cars waiting on its side of the 
//  bridge and less than three cars on the bridge --OR--  
//  the bridge is clear and no-one is waiting anywhere.


void Bridge::ArriveBridge(direction d) {
  bridgeLock->Acquire();

  DEBUG('p', "The %s car arrives from the %s\n",
	currentThread->getName(), d ? "east" : "west");

  if (isWaitingCar(d) || numCarsOnBridge == 3 ||
      (d != currentDirection && (numCarsOnBridge ||
				 isWaitingCar(SwitchD(d))))) {
    WaitInLine(d);

  } else { 
    currentDirection = d;
    numCarsOnBridge++;
    ASSERT(numCarsOnBridge <= 3);
  }
  bridgeLock->Release();
  CrossBridge(d);
}

void Bridge::CrossBridge(direction d) {
  DEBUG('p', "The %s car crosses bridge going %s\n",
	 currentThread->getName(), d ? "west" : "east");
  ExitBridge(d);
}



//  ExitBridge controls the number of cars on the bridge.
//  The waiting queues on either side are signaled as 
//  appropriate.


void Bridge::ExitBridge(direction d) {
  int i;
  bridgeLock->Acquire();
  numCarsOnBridge--;

  DEBUG('p', "The %s car leaves going %s\n",
	currentThread->getName(), d ? "west" : "east");

  if (numCarsOnBridge < 3 && isWaitingCar(d)) {
    for(i = 0; 1 < 3 && i < NumWaiting(d) && i < 3 - numCarsOnBridge; i++) {
      d ? safeToCrossGoingWest->Signal(bridgeLock)
	: safeToCrossGoingEast->Signal(bridgeLock);
      numCarsOnBridge++;
      if(d) numWaitingGoingWest--;
      else numWaitingGoingEast--;
    }
  } else if(!numCarsOnBridge && !isWaitingCar(d)) {
    for(i = 0; i < 3 && i < NumWaiting(SwitchD(d)); i++) {
      d ? safeToCrossGoingEast->Signal(bridgeLock)
	: safeToCrossGoingWest->Signal(bridgeLock);
      numCarsOnBridge++;
      if(!d) numWaitingGoingWest--;
      else numWaitingGoingEast--;
    }
    d = SwitchD(d);
  }
  ASSERT(numCarsOnBridge <= 3);
  bridgeLock->Release();
}



// Utility functions


void Bridge::WaitInLine(direction d) {
  if (d) {
    numWaitingGoingWest++;
    safeToCrossGoingWest->Wait(bridgeLock);
  } else { 
    numWaitingGoingEast++;
    safeToCrossGoingEast->Wait(bridgeLock);
  }
}

bool Bridge::isWaitingCar(direction d) {
  if(d) return numWaitingGoingWest ? TRUE : FALSE;
  else  return numWaitingGoingEast ? TRUE : FALSE;
}

int Bridge::NumWaiting(direction  d) {
  if(d) return numWaitingGoingWest;
  return numWaitingGoingEast;
}

direction SwitchD(direction d) {
  if(d) return EAST;
  return WEST;
}



