 // elevator.cc --All the code for HW1 #4




//                 ***GENERAL IDEA***
// 
//  A building constists of an array of floors. Each floor 
//  has two queues (condition vars.) waitToGoUp and 
//  waitToGoDown it also keeps track of how many people are
//  in each queue (numGoingUp, numGoingDown).  The elevator 
//  has two condition vars: inElevator where people wait for 
//  their floor once they've gotten on and okToCloseDoor where 
//  the elevator waits unitl it can leave the floor.  The 
//  elevator also remebers how many people need to get on
//  (numRidersToGetOn) and how many people inside are waiting
//  to get off on each floor (numRidersToGetOff). There is a 
//  lock on each floor and a lock on the elevator.  People get on
//  the elevator only when it is going in the appropriate 
//  direction.



extern "C" {
#include <stdlib.h>
#include <time.h>
}

#include "elevator.hpp"
#include "alarm.hpp"
#include "system.hpp"
#include "stats.hpp"

static void StartStudent(int name);
void StartElevator(int trips);

static Building *Evans = new Building(11);
static Elevator *CarOne = new Elevator(11);

Building::Building(int howManyFloors) {
  numFloors = howManyFloors;
  floors = new Floor*[numFloors + 1];
  for (int i = 0; i <= numFloors; i++)
    floors[i] = new Floor(i);
}

Floor::Floor(int num) {
  waitToGoUp = new Condition("waitToGoUp");
  waitToGoDown = new Condition("waitToGoDown");
  floorLock = new Lock("floorLock");
  numGoingUp = numGoingDown = 0;
}

Elevator::Elevator(int floors) {
  numFloors = floors;
  inElevator = new Condition("inElevator");
  okToCloseDoor = new Condition("okToCloseDoor");
  doorLock = new Lock("elevatorDoorLock");
  numRidersToGetOff = new int[numFloors + 1];
  currentFloor = numRidersToGetOn = 0;
  direction = Up;
}

Person::Person(int nameNum) {
  name = nameNum;
}


//  A person first acquires the floorLock.  She then waits on
//  the appropriate condition var until the elevator arrives in
//  her direction.  Immediately upon waking she releases the floor
//  Lock and acquires the doorLock (elevator).  If she is the last
//  one to get on or off she signals the elevator.  She then
//  blocks on inElevator.  When ever the elevator stops she wakes
//  up and checks to see if this is the right floor.  If so she
//  gets off and if she is either the last on or off she signals
//  the elevator.



void Person::GoingFromTo(int atFloor, int toFloor) {
  Lock *floorLock = Evans->floors[atFloor]->floorLock;

  floorLock->Acquire();
  DEBUG('p', "Student %d waiting at floor %d to go to floor %d.\n",
     name, atFloor, toFloor);

  if (atFloor < toFloor) {
    Evans->floors[atFloor]->numGoingUp++;
    Evans->floors[atFloor]->waitToGoUp->Wait(floorLock);
  } else { 
    Evans->floors[atFloor]->numGoingDown++;
    Evans->floors[atFloor]->waitToGoDown->Wait(floorLock);
  }
  
  floorLock->Release();

  CarOne->doorLock->Acquire();
  DEBUG('p', "Student %d getting on elevator.\n", name);

  CarOne->numRidersToGetOn--;

  ((CarOne->numRidersToGetOff)[toFloor])++;

  if (CarOne->isTimeToClose())
    CarOne->okToCloseDoor->Signal(CarOne->doorLock);

  while(toFloor != CarOne->currentFloor)
    CarOne->inElevator->Wait(CarOne->doorLock);
  
  DEBUG('p', "Student %d getting off elevator.\n", name);

  ASSERT(toFloor == CarOne->currentFloor);
  ((CarOne->numRidersToGetOff)[toFloor])--;

  if (CarOne->isTimeToClose())
    CarOne->okToCloseDoor->Signal(CarOne->doorLock);

  CarOne->doorLock->Release();
  
  currentThread->Finish();
}



//  The elevator holds its own lock between floors.
//  When the elevator arrives at a floor it wakes up all of the 
//  people blocked on inElevator.  Those who are not ready to get
//  off go back to sleep.  It then releases its own lock, acquires
//  the floorLock, determines how many people are blocked on the
//  queue for the appropriate direction, releases them and resets
//  either numGoingUp or numGoingDown.  It then release the floor
//  Lock and acquires its own lock.  it then blocks on okToClose
//  Door until all the people that want to get in and out do.



void Elevator::TurnOn(int trips) {
  Floor *thisFloor;
  doorLock->Acquire();

  for (int i = 0; i < trips; i++) {
    thisFloor = Evans->floors[currentFloor];

    if (currentFloor == 0)
      direction = Up;
    else if (currentFloor == Evans->numFloors)
      direction = Down;

    inElevator->Broadcast(doorLock);
 
    doorLock->Release();
    
    thisFloor->floorLock->Acquire();

    DEBUG('p', "Elevator arriving at floor %d, Going %s.\n",
       currentFloor, direction == Up ? "Up" : "Down");
    
    if (direction == Up) {
      numRidersToGetOn = thisFloor->numGoingUp;
      thisFloor->numGoingUp = 0;
      thisFloor->waitToGoUp->Broadcast(thisFloor->floorLock);
    } else {
      numRidersToGetOn = thisFloor->numGoingDown;
      thisFloor->numGoingDown = 0;
      thisFloor->waitToGoDown->Broadcast(thisFloor->floorLock);
    }
    
    thisFloor->floorLock->Release();
    
    doorLock->Acquire();
    
    while (!isTimeToClose())
      okToCloseDoor->Wait(CarOne->doorLock);
    
    DEBUG('p', "Elevator departing floor %d, Going %s.\n",
       currentFloor, direction == Up ? "Up" : "Down");
    
    if (direction == Up)
      currentFloor++;
    else currentFloor--;
    
    alarm->GoToSleepUntil(stats->totalTicks + 100);
  }

  doorLock->Release();
  currentThread->Finish();
}



bool Elevator::isTimeToClose() {
  if(numRidersToGetOn == 0 &&
     numRidersToGetOff[currentFloor] == 0)
    return TRUE;
  else return FALSE;
}



void StartElevatorSimulation() {

  Thread *t = new Thread("forked thread");

  t->Fork((VoidFunctionPtr)StartElevator, 50);

  for(int i = 0; i < 20; i++) {
    t = new Thread("forked thread");
    t->Fork((VoidFunctionPtr)StartStudent, i+1);
  }
}



void StartElevator(int trips) {
  CarOne->TurnOn(trips);
}


void StartStudent(int name) {

 /* srand((time(NULL) * name) % 1007);*/
  int toFloor = rand() % 12;

  alarm->GoToSleepUntil(stats->totalTicks + 400);

  /*srand((time(NULL) * name) % 3001);    */
  int atFloor = rand() % 12;
  
  Person *student = new Person(name);
  
  student->GoingFromTo(atFloor, toFloor);
}



